require(quantstrat)
require(quantmod)
require(blotter)

# Similar to sma.monthly.r, with the added twist that the trading is
# performed on day by day basis. This way, we have better control over
# stop orders.

Sys.setenv(TZ="UTC")

# Setup
strategyName   = "SMA"
portfolioName  = "SMA"
accountName    = "SMA"
symbol         = "SPY"
symbolFile     = "SPY.bin"
# initDate       = "2008-01-01"
initDate       = "1993-01-01"
endDate        = "2012-12-31"
# endDate        = "1998-12-31"
initEq         = 100000
orderQty       = initEq
goShort        = FALSE
goShort        = TRUE
stopLoss       = NULL
# stopLoss       = 0.02
stopTrailing = NULL
stopTrailing   = 0.03

# verbose = FALSE
verbose = TRUE

orderType = "market"
enterType = "enter"
exitType = "exit"

suppressWarnings(rm("order_book.SMA", pos=.strategy))
suppressWarnings(rm("account.SMA", pos=.blotter))
suppressWarnings(rm("portfolio.SMA", pos=.blotter))

# Define the strategy
strategy(strategyName, store=TRUE)

# A helper to compute an indicator of a more coarse scale, and then
# map it to the original scale.
#
# data.func looks odd, however, to.period returns an OHLC, which we need to make work
# for various indicators, some of them taking only prices as parameters, some of them'
# HLC. The solution is to pass the function to convert OHLC to the data for the indicator
# as a parameter.
stretched.indicator = function(x, indicator="SMA", on="months", k=1, data.func="Cl", ...) {
   period.ends = endpoints(x, on=on, k=k)
   
   # Get the aggregated series
   x.monthly = do.call(data.func, list(to.period(x, period=on, k=k, indexAt="endof")))
   
   # Compute the indicator
   indicator = do.call(indicator, c(list(x.monthly), ...))
   
   # Initialize the result
   res = matrix(NA, nrow=NROW(x), ncol=NCOL(indicator))
   
   len = NROW(period.ends)
   ii = 2
   while(ii < len) {
      # Assignment of vector to matrix is done column by column, hence, we need to build similar matrix
      # on the right hand side.
      res[period.ends[ii]:(period.ends[ii+1]-1),] = matrix(rep(indicator[ii-1,], each=period.ends[ii+1]-period.ends[ii]), ncol=NCOL(indicator))
      ii = ii + 1
   }
   
   # The last point
   res[period.ends[ii],] = indicator[ii-1,]
   
   # Reclass and setup the column names
   res = reclass(res, x)
   colnames(res) = colnames(indicator)
   
   return(res)
}

# One indicator, MACD gives us the relationship between the SMA(10) and the price (which is SMA(1)).
# We compute the monthly relation between the price and the SMA(10), then we map the monthly indicator
# to the daily series.
stretched.macd = function(x) {
   return(stretched.indicator(x=x, indicator="MACD", nFast=1, nSlow=10, maType="SMA"))
}

# The indicator is added manually, see the code where the SPY is loaded
#add.indicator(
#      strategyName,
#      name="stretched.macd",
#      label="SMACD",
#      arguments=list(x=quote(mktdata[,c()])))

# Two signals
add.signal(
      strategyName,
      name="sigThreshold",
      arguments=list(column="macd",
                     relationship="gte",
                     threshold=0,
                     cross=TRUE),
      label="macd.gte.zero")
add.signal(
      strategyName,
      name="sigThreshold",
      arguments=list(column="macd",
                     relationship="lt",
                     threshold=0,
                     cross=TRUE),
      label="macd.lt.zero")

# Notice, that for the indicator we use the "Close", but for trading, we
# use "Adjusted" to account for dividends.

# Long entry
add.rule(
      strategyName,
      name='ruleSignal',
      arguments=list(sigcol="macd.gte.zero",
                     sigval=TRUE,
                     orderqty=1000,
                     ordertype="market",
                     pricemethod="market",
                     orderside="long",
                     prefer="Open",
                     orderset="long.chain"),
      type="enter",
      label="enter.long")

# Long exit
add.rule(
      strategyName,
      name="ruleSignal",
      arguments=list(sigcol="macd.lt.zero",
                     sigval=TRUE,
                     orderqty="all",
                     ordertype="market",
                     pricemethod="market",
                     orderside="long",
                     prefer="Open",
                     orderset="long.chain"),
      type="exit",
      label="exit.long")

if(goShort) {
   # Short entry
   add.rule(
         strategyName,
         name='ruleSignal',
         arguments=list(sigcol="macd.lt.zero",
                        sigval=TRUE,
                        orderqty=-1000,
                        ordertype="market",
                        pricemethod="market",
                        orderside="short",
                        prefer="Open",
                        orderset="short.chain"),
         type="enter",
         label='enter.short')

   # Short exit
   add.rule(
         strategyName,
         name='ruleSignal',
         arguments=list(sigcol="macd.gte.zero",
                        sigval=TRUE,
                        orderqty="all",
                        ordertype="market",
                        pricemethod="market",
                        orderside="short",
                        prefer="Open",
                        orderset="short.chain"),
         type="exit",
         label='exit.short')
}

if(!is.null(stopLoss)) {
   # Long stop loss
   add.rule(
         strategyName,
         name="ruleSignal",
         arguments=list(sigcol="macd.gte.zero",
                        sigval=TRUE,
                        replace=FALSE,
                        orderqty="all",
                        ordertype="stoplimit",
                        orderside="long",
                        tmult=TRUE,
                        threshold=stopLoss,
                        orderset="long.chain"),  
         type="chain",
         parent="enter.long",
         label="stop.loss.long",
         storefun=FALSE)
   
   if(goShort) {
      # Short stop loss
      add.rule(
         strategyName,
         name="ruleSignal",
         arguments=list(sigcol="macd.lt.zero",
                        sigval=TRUE,
                        replace=FALSE,
                        orderqty="all",
                        ordertype="stoplimit",
                        orderside="short",
                        tmult=TRUE,
                        threshold=stopLoss,
                        orderset="short.chain"),  
         type="chain",
         parent="enter.short",
         label="stop.loss.short",
         storefun=FALSE)
   }
}

if(!is.null(stopTrailing)) {
   # Long stop trailing
   add.rule(
      strategyName,
      name="ruleSignal",
      arguments=list(sigcol="macd.gte.zero",
                     sigval=TRUE,
                     replace=FALSE,
                     orderside="long",
                     ordertype="stoptrailing",
                     tmult=TRUE,
                     threshold=stopTrailing,
                     orderqty="all",
                     orderset="long.chain"),  
      type="chain",
      parent="enter.long",
      label="stop.trailing.long",
      storefun=FALSE)
   
   if(goShort) {
      # Short stop trailing
      add.rule(
         strategyName,
         name="ruleSignal",
         arguments=list(sigcol="macd.lt.zero",
                        sigval=TRUE,
                        replace=FALSE,
                        orderside="short",
                        ordertype="stoptrailing",
                        tmult=TRUE,
                        threshold=stopTrailing,
                        orderqty="all",
                        orderset="short.chain"),  
         type="chain",
         parent="enter.short",
         label="stop.trailing.short",
         storefun=FALSE)
   }
}

currency("USD")
stock(symbol, currency="USD", multiplier=1)

# Load the data and convert to monthly
load(file=symbolFile)

# Compute the indicator using the original OHLC series. For the trading,
# we need to adjust them to account for dividends, hence, compute the
# indicator while we have the original without bothering to do add.indicator.
# spy.macd = na.trim(lag(stretched.macd(OHLC(SPY))$macd,-1))
spy.macd = stretched.macd(OHLC(SPY))$macd
colnames(spy.macd) = c("SPY.macd")

# Adjust the SPY
SPY = adjustOHLC(SPY, use.Adjusted=TRUE)
# SPY["1994-09-01","SPY.Low"] = 30

# Merge with the indicator
SPY = merge(SPY, spy.macd, all=FALSE)

# Apply the endDate
SPY = SPY[paste(initDate, "/", endDate, sep="")]

# Portfolio, account and order book
initPortf(portfolioName, symbols=symbol, initDate=initDate)
initAcct(accountName, portfolios=portfolioName, initDate=initDate, initEq=initEq)
initOrders(portfolio=portfolioName, initDate=initDate)

out = try(applyStrategy(strategy=strategyName, portfolios=portfolioName))

updatePortf(portfolioName, Symbols=symbol, Dates=paste("/", as.Date(Sys.time()), sep=""))
updateAcct(portfolioName)

getPNL = function() as.numeric(tradeStats(portfolioName, symbol)$Net.Trading.PL)
getPerTradeStats = function() perTradeStats(portfolioName, symbol)[,c(1,2,3,4,5,7,10,11,12)]

print(paste( "Net Trading PnL: ", getPNL(), sep=""))
print(getPerTradeStats())
